class ArtistsController < ApplicationController
layout 'test'
  before_filter :authenticate_user!, :only => [:index, :show, :new, :create, :edit, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]

  # GET /artists
  # GET /artists.json
  def index
    @page = Artist.page params[:page]
    @page_size = Artist.page_size params[:page_size]
    @artists = Artist.list(@page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render :json => @artists.to_json(Artist.list_json_opt) }
    end
  end

  # GET /artists/1
  # GET /artists/1.json
  def show
    @artist = Artist.show(params[:id], @author)

    respond_to do |format|
      format.html # show.html.erb
      format.json { render :json => @artist.to_json(Artist.show_json_opt) }
    end
  end

  def count
    @artist = {:count => Artist.visible_count}
    respond_to do |format|
      format.json { render json: @artist.to_json }
    end
  end
  
  def list
    @artists = Artist.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @artists }
    end
  end

  def browse
    @artist = Artist.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @artist }
    end
  end

  # GET /artists/new
  # GET /artists/new.json
  def new
    @artist = Artist.new
    @artist.supply_default 

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @artist.to_json(Artist.show_json_opt) }
    end
  end

  # GET /artists/1/edit
  def edit
    @artist = Artist.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  # POST /artists
  # POST /artists.json
  def create
    @artist = Artist.new()
    @artist.supply_default 
    @artist.attributes = params[:artist]
    @artist.overwrite @author
    respond_to do |format|
      if @artist.save
        format.html { redirect_to @artist, notice: 'Artist was successfully created.' }
        format.json { render json: @artist.to_json(Artist.show_json_opt), status: :created, location: @artist }
      else
        format.html { render action: "new" }
        format.json { render json: @artist.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /artists/1
  # PUT /artists/1.json
  def update
    @artist = Artist.edit(params[:id], @author)
    @artist.attributes = params[:artist]
    @artist.overwrite @author

    respond_to do |format|
      if @artist.save
        format.html { redirect_to @artist, notice: 'Artist was successfully updated.' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @artist.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /artists/1
  # DELETE /artists/1.json
  def destroy
    @artist = Artist.edit(params[:id], @author)
    @artist.destroy

    respond_to do |format|
      format.html { redirect_to artists_url }
      format.json { head :ok }
    end
  end
end
