class ComicsController < ApplicationController
  layout 'test' if Pettanr::TestLayout
  if Const.run_mode == 0
    before_filter :authenticate_user!, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_user!, :only => [:top, :index, :show, :play, :new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def top
    @new_comics = Comic.find(:all, 
      :include => :author, :conditions => ['visible > 0'], :order => 'updated_at desc', :limit => 5
    )
    @new_pictures = OriginalPicture.list @artist.id

    respond_to do |format|
      format.html # index.html.erb
    end
  end
  
  # GET /comics
  # GET /comics.json
  def index
    @page = Comic.page params[:page]
    @page_size = Comic.page_size params[:page_size]
    @comics = Comic.list(@page, @page_size)
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @comics.to_json(Comic.list_json_opt) }
    end
  end

  # GET /comics/1
  # GET /comics/1.json
  def show
    @comic = Comic.show(params[:id], @author)

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @comic.to_json(Comic.show_json_opt) }
    end
  end

  def count
    @comic = {:count => Comic.visible_count}
    respond_to do |format|
      format.json { render json: @comic.to_json }
    end
  end
  
  def list
    @comics = Comic.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @comics }
    end
  end

  def browse
    @comic = Comic.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @comic }
    end
  end
  
  # GET /comics/new
  # GET /comics/new.js
  def new
    @comic = Comic.new
    @comic.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @comic.to_json(Comic.show_json_opt) }
    end
  end

  # GET /comics/1/edit
  # GET /comics/1.js/edit
  def edit
    @comic = Comic.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  # POST /comics
  # POST /comics.json
  def create
    @comic = Comic.new
    @comic.supply_default 
    @comic.attributes = params[:comic]
    @comic.overwrite @author

    respond_to do |format|
      if @comic.save
        format.html { redirect_to @comic, notice: 'Comic was successfully created.' }
        format.json { render json: @comic.to_json(Comic.show_json_opt), status: :created, location: @comic }
      else
        format.html { render action: "new" }
        format.json { render json: @comic.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /comics/1
  # PUT /comics/1.json
  def update
    @comic = Comic.edit(params[:id], @author)
    @comic.attributes = params[:comic]
    @comic.overwrite @author
    respond_to do |format|
      if @comic.save
        format.html { redirect_to @comic, notice: 'Comic was successfully updated.' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @comic.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /comics/1
  # DELETE /comics/1.json
  def destroy
    @comic = Comic.edit(params[:id], @author)
    @comic.destroy
    respond_to do |format|
      format.html { redirect_to comics_url }
      format.json { head :ok }
    end
  end
end
