class PanelsController < ApplicationController
  layout 'test' if Pettanr::TestLayout
  before_filter :authenticate_user!, :only => [:index, :show, :new, :edit, :create, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]

  public
  
  # GET /panels
  # GET /panels.json
  def index
    @page = Panel.page params[:page]
    @page_size = Panel.page_size params[:page_size]
    @panels = Panel.list(@page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render text: '[' + @panels.map {|i| i.panel_elements_as_json }.join(',') + ']' }
    end
  end

  # GET /panels/1
  # GET /panels/1.json
  def show
    @panel = Panel.show(params[:id], @author)

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @panel.panel_elements_as_json }
      format.jsonp {
        render :json => "callback(" + @panel.panel_elements_as_json + ");"
      }
    end
  end

  def count
    @panel = {:count => Panel.visible_count}
    respond_to do |format|
      format.json { render json: @panel.to_json }
    end
  end
  
  def list
    @panels = Panel.all :order => 'updated_at'

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @panels }
    end
  end

  def browse
    @panel = Panel.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
    end
  end

  def new
    @panel = Panel.new
    @panel.supply_default

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @panel.panel_elements_as_json }
    end
  end

  def edit
    @panel = Panel.edit(params[:id], @author)
    respond_to do |format|
      format.html
      format.js
    end
  end

  # POST /panels
  # POST /panels.json
  def create
    @panel = Panel.new
    @panel.supply_default
    jsn = nil
    if params[:json]
      jsn = JSON.parse_no_except(params[:json])
    end
    @prm = params[:panel] || jsn
    respond_to do |format|
      if @panel.store @prm, @author
        format.html { redirect_to @panel, notice: 'Panel was successfully created.' }
        format.json { render json: @panel.panel_elements_as_json, status: :created, location: @panel }
      else
        format.html { render action: "new" }
        format.json { render json: @panel.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /panels/1
  # PUT /panels/1.json
  def update
    @panel = Panel.edit(params[:id], @author)
    jsn = nil
    if params[:json]
      jsn = JSON.parse(params[:json])
    end
    @prm = params[:panel] || jsn
    respond_to do |format|
      if @panel.store @prm, @author
        format.html { redirect_to @panel, notice: 'Panel was successfully updated.' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @panel.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /panels/1
  # DELETE /panels/1.json
  def destroy
    @panel = Panel.edit(params[:id], @author)
    respond_to do |format|
      Panel.transaction do
        @panel.destroy
        format.html { redirect_to panels_url }
        format.json { head :ok }
      end
    end
  end
  
end
