# -*- encoding: utf-8 -*-
#実素材
require 'spec_helper'

describe PicturesController do
  before do
    FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
  end

  describe '単体表示に於いて' do
    before do
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      sign_in @user
      Picture.stub(:show).with(@p.id.to_s, @author).and_return(@p)
    end
    context 'つつがなく終わるとき as json' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @p.id, :format => :json
        response.should be_success
      end
      it '実素材モデルに単体取得を問い合わせている' do
        Picture.should_receive(:show).exactly(1)
        get :show, :id => @p.id, :format => :json
      end
      it '@pictureにアレを取得している' do
        get :show, :id => @p.id, :format => :json
        assigns(:picture).id.should eq(@p.id)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @p.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @p.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @p.id, :format => :json
          json = JSON.parse response.body
          json["revision"].should_not be_nil
          json["ext"].should match(/png/)
        end
      end
      #画像送信では、send_dataにスタブをおいてテストしたいが、ここに噛ませると
      #renderが働かず、エラーとなってしまう。そこで、素材のファイル取得部分に
      #スタブをおいてsend_dataがデータを返す体裁でテストする。
      context 'png形式' do
        it '画像モデルに画像データを問い合わせる' do
          Picture.any_instance.should_receive(:restore).exactly(1)
          get :show, :id => @p.id, :format => :png
        end
        it '画像を送信する' do
          Picture.any_instance.stub(:restore).and_return('aaa')
          get :show, :id => @p.id, :format => :png
          response.body.should eq 'aaa'
        end
      end
      context 'gif形式' do
        it '画像モデルに画像データを問い合わせる' do
          Picture.any_instance.should_receive(:restore).exactly(1)
          get :show, :id => @p.id, :format => :gif
        end
        it '画像を送信する' do
          Picture.any_instance.stub(:restore).and_return('bbb')
          get :show, :id => @p.id, :format => :gif
          response.body.should eq 'bbb'
        end
      end
      context 'jpeg形式' do
        it '画像モデルに画像データを問い合わせる' do
          Picture.any_instance.should_receive(:restore).exactly(1)
          get :show, :id => @p.id, :format => :jpeg
        end
        it '画像を送信する' do
          Picture.any_instance.stub(:restore).and_return('ccc')
          get :show, :id => @p.id, :format => :jpeg
          response.body.should eq 'ccc'
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @p.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @p.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @p.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @p.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象素材がないとき' do
      before do
        Picture.unstub(:show)
      end
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
    context '他人の素材を見ようとしたとき' do
      before do
        Picture.stub(:show).and_return(@p)
        Picture.any_instance.stub(:own?).with(any_args()).and_return(false)
      end
      context 'html形式' do
        it '例外403 forbiddenを返す' do
          lambda{
            get :show, :id => @p.id
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
      context 'json形式' do
        it '例外403 forbiddenを返す' do
          lambda{
            get :show, :id => @p.id, :format => :json
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
    end
=end
  end

  describe 'クレジット表示に於いて' do
    before do
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      sign_in @user
      Picture.stub(:show).with(@p.id.to_s, @author).and_return(@p)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :credit, :id => @p.id
        response.should be_success
      end
      it '実素材モデルに単体取得を問い合わせている' do
        Picture.should_receive(:show).exactly(1)
        get :credit, :id => @p.id
      end
      it '@pictureにアレを取得している' do
        get :credit, :id => @p.id
        assigns(:picture).id.should eq(@p.id)
      end
      context 'html形式' do
        it 'creditテンプレートを描画する' do
          get :credit, :id => @p.id
          response.should render_template("credit")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :credit, :id => @p.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :credit, :id => @p.id, :format => :json
          json = JSON.parse response.body
          json["revision"].should_not be_nil
          json["ext"].should match(/png/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :credit, :id => @p.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :credit, :id => @p.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :credit, :id => @p.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :credit, :id => @p.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象素材がないとき' do
      before do
        Picture.unstub(:show)
      end
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
=end
  end

end
