# -*- encoding: utf-8 -*-
require 'spec_helper'
#コマの色背景

describe PanelColor do
  before do
    FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = @other_user.author
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end
  
  describe '検証に於いて' do
    before do
      @pc = FactoryGirl.build :panel_color, :panel_id => @panel.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @pc.z = 1
        @pc.code = 0
        @pc.should be_valid
      end
      it '上限データが通る' do
        @pc.z = 99999
        @pc.code = 0xffffff
        @pc.should be_valid
      end
    end
    
    
    context 'panel_idを検証するとき' do
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it 'テストデータの確認' do
        @pc.panel_id = @panel.id
        @pc.should be_valid
      end
      it '数値でなければ失敗する' do
        @pc.panel_id = 'a'
        @pc.should_not be_valid
      end
    end
    context 'codeを検証するとき' do
      it 'テストデータの確認' do
        @pc.code = 0xffffff
        @pc.should be_valid
      end
      it 'nullなら失敗する' do
        @pc.code = nil
        @pc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pc.code = 'a'
        @pc.should_not be_valid
      end
      it '負なら失敗する' do
        @pc.code = -1
        @pc.should_not be_valid
      end
      it '24bit colorでなければ失敗する' do
        @pc.code = 0x1000000
        @pc.should_not be_valid
      end
    end
    context 'zを検証するとき' do
      it 'テストデータの確認' do
        @pc.z = 1
        @pc.should be_valid
      end
      it 'nullなら失敗する' do
        @pc.z = nil
        @pc.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pc.z = 'a'
        @pc.should_not be_valid
      end
      it '負なら失敗する' do
        @pc.z = -1
        @pc.should_not be_valid
      end
      it '負なら失敗する' do
        @pc.z = 0
        @pc.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'defined' do
      @pc = FactoryGirl.create :panel_color, :panel_id => @panel.id
      @pc.supply_default
    end
  end
  
  describe '上書き補充に於いて' do
    it 'defined' do
      @pc = FactoryGirl.create :panel_color, :panel_id => @panel.id
      @pc.overwrite
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @pc = FactoryGirl.create :panel_color, :panel_id => @panel.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        PanelColor.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        PanelColor.page().should eq 1
      end
      it '0以下の場合は1になる' do
        PanelColor.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        PanelColor.page_size('7').should eq 7
      end
      it 'nilの場合はPanelColor.default_page_sizeになる' do
        PanelColor.page_size().should eq PanelColor.default_page_size
      end
      it '0以下の場合はPanelColor.default_page_sizeになる' do
        PanelColor.page_size('0').should eq PanelColor.default_page_size
      end
      it 'PanelColor.max_page_sizeを超えた場合はPanelColor.max_page_sizeになる' do
        PanelColor.page_size('1000').should eq PanelColor.max_page_size
      end
    end
    it 'リストを返す' do
      pl = PanelColor.list
      pl.should eq [@pc]
    end
    it '時系列で並んでいる' do
      #公開コマなら(他人のコマであっても)含んでいる
      opl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :panel_color, :panel_id => opl.id, :z => 2, :updated_at => Time.now + 100
      pl = PanelColor.list
      pl.should eq [npl, @pc]
    end
    it '非公開のコマの景色は含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :panel_color, :panel_id => hpl.id, :updated_at => Time.now + 100
      pl = PanelColor.list
      pl.should eq [@pc]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @pc2 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 1, :updated_at => Time.now + 100
        @pc3 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 200
        @pc4 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 3, :updated_at => Time.now + 300
        @pc5 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 4, :updated_at => Time.now + 400
        PanelColor.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = PanelColor.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = PanelColor.list(1)
        pl.should eq [@pc5, @pc4]
      end
      it 'page=2なら中間2件を返す' do
        pl = PanelColor.list(2)
        pl.should eq [@pc3, @pc2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = PanelColor.list(3)
        pl.should eq [@pc]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @pc2 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 1, :updated_at => Time.now + 100
        @pc3 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 200
        @pc4 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 3, :updated_at => Time.now + 300
        @pc5 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 4, :updated_at => Time.now + 400
      end
      it '通常は全件(5件)を返す' do
        r = PanelColor.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe 'list関連テーブルプションに於いて' do
    it 'includeキーを含んでいる' do
      r = PanelColor.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = PanelColor.list_opt[:include]
      r.should have(1).items
    end
    it 'コマを含んでいる' do
      r = PanelColor.list_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = PanelColor.list_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
  end
  describe 'json一覧出力オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = PanelColor.list_json_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = PanelColor.list_json_opt[:include]
      r.should have(1).items
    end
    it 'コマを含んでいる' do
      r = PanelColor.list_json_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = PanelColor.list_json_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
  end
  
  describe '自分のコマで使った色一覧取得に於いて' do
    before do
      @pc = FactoryGirl.create :panel_color, :panel_id => @panel.id
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        PanelColor.stub(:list_opt).with(any_args).and_return({:include => :panel})
        PanelColor.should_receive(:list_opt).with(any_args).exactly(1)
        r = PanelColor.mylist @author
      end
    end
    it 'リストを返す' do
      pl = PanelColor.mylist @author
      pl.should eq [@pc]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 100
      pl = PanelColor.mylist @author
      pl.should eq [npl, @pc]
    end
    it '他人のコマの景色は公開でも含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :panel_color, :panel_id => hpl.id
      pl = PanelColor.mylist @author
      pl.should eq [@pc]
    end
    it '自分のコマの景色は非公開でも含んでいる' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :panel_color, :panel_id => hpl.id, :z => 2, :updated_at => Time.now + 100
      pl = PanelColor.mylist @author
      pl.should eq [npl, @pc]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @pc2 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 1, :updated_at => Time.now + 100
        @pc3 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 200
        @pc4 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 3, :updated_at => Time.now + 300
        @pc5 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 4, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = PanelColor.mylist @author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = PanelColor.mylist(@author, 1, 2)
        c.should eq [@pc5, @pc4]
      end
      it 'page=2なら中間2件を返す' do
        c = PanelColor.mylist(@author, 2, 2)
        c.should eq [@pc3, @pc2]
      end
      it 'page=3なら先頭1件を返す' do
        c = PanelColor.mylist(@author, 3, 2)
        c.should eq [@pc]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @pc2 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 1, :updated_at => Time.now + 100
        @pc3 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 2, :updated_at => Time.now + 200
        @pc4 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 3, :updated_at => Time.now + 300
        @pc5 = FactoryGirl.create :panel_color, :panel_id => @panel.id, :z => 4, :updated_at => Time.now + 400
      end
      it '通常は全件(5件)を返す' do
        r = PanelColor.mylist @author, 5, 0
        r.should have(5).items 
      end
    end
  end
  
end
